unit NVIAC_Button_Unit;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,NVIAC_BASIS,nviac_textlines_unit,
  nviac_bitmap_unit;
type
  TNVIAC_Button_Press_State =(nviacbpsUp,nviacbpsDown);
  TNVIAC_Button_Form =(nviacbfRect,nviacbfElliptic,nviacbfEllipDiag,nviacbfEllipRevDiag,nviacbfOctaRect);
  TNVIAC_Button = class(TCustomControl,INVIAC_SEMITRANSPARENCE)
  private
    ABasis : TNVIAC_BASIS;
    ACurrRect : TRect;
    AImage : TBitmap;
    FSemiTransparence: Byte;
    FPressState: TNVIAC_Button_Press_State;
    FButtonForm: TNVIAC_Button_Form;
    FSkinImageList : TImageList;
    FSkinApply: TNVIAC_Bitmap_Apply;
    FMouseDown : Boolean;
    function GetColor: TColor;
    function GetEnabled1: boolean;
    function GetFont: TFont;
    function GetHeight: integer;
    function GetLeft: integer;
    function GetText: TCaption;
    function GetTop: integer;
    function GetVisible: boolean;
    function GetWidth: integer;
    procedure SetColor(Value: TColor);
    procedure SetEnabled1(Value: boolean);
    procedure SetFont(Value: TFont);
    procedure SetHeight(Value: integer);
    procedure SetLeft(Value: integer);
    procedure SetText(Value: TCaption);
    procedure SetTop(Value: integer);
    procedure SetVisible(Value: boolean);
    procedure SetWidth(Value: integer);
    procedure SizedHiding;
    procedure SizedShowing;
    procedure FontIsChanged(Sender: TObject);
    procedure GetBaseBitmap;
    procedure SetSemiTransparence(Value: Byte);
    procedure SetPressState(Value: TNVIAC_Button_Press_State);
    procedure WMLBUTTONDBLCLK(var message : TMessage);message WM_LBUTTONDBLCLK; 
    procedure WMLBUTTONDOWN(var message : TMessage);message WM_LBUTTONDOWN;
    procedure WMLBUTTONUP(var message : TMessage);message WM_LBUTTONUP;
    procedure WMKEYDOWN(var message : TMessage);message WM_KEYDOWN;
    procedure WMKEYUP(var message : TMessage);message WM_KEYUP;
    procedure WMKILLFOCUS(var message : TMessage);message WM_KILLFOCUS;
    procedure WMSETFOCUS(var message : TMessage); message WM_SETFOCUS;
    procedure DefineCurrentRegion;
    procedure SetButtonForm(Value: TNVIAC_Button_Form);
    procedure SetSkinApply(Value: TNVIAC_Bitmap_Apply);
    procedure SetSkinImageList(Value: TImageList);
    function GetSkinImageList: TImageList;
    
  protected
    procedure Paint;override;
    procedure CreateHandle;override;
  public
    procedure ReintroduceBasis(ANewBasis : TNVIAC_BASIS);//Get new basis if it changes
    procedure TryRegisterSemiQueque;//Register in semi queue
    procedure TryUnregisterSemiQueque;//Unregister
    function GetFullBitmap: TBitmap;//This is full bitmap that we can look at form
    function GetCurrentSize : TSize;//Size of control
    function GetCurrentBitmap: TBitmap;//Bitmap to process semi
    function GetCurrentOffset: TPoint;//position of control by form client rect
    function GetCurrentRegion: HRGN;//Region of control trimming (where to paint - will painted only here on back)
    function GetSemiClass : Byte;//1 - Alpha,2 - bitmap
    function GetAlpha : Byte;//alpha
    function GetAlphaBitmap : TBitmap;//alpha bitmap sizes=Current Bitmap
    procedure ContinueBitmap;//Requiring to draw the bitmap after semi to full image
    procedure TryPaint;//Requiring to redraw the control (Repaint - usually, some complex for Controls having others controls)
    procedure SetCurrRect(ARect : TRect);//This is setting of rect where and only where you can draw Current/Continue bitmap
    function GetSemiControl: TControl;//Must Return someself t.i. -> Result:=Self;
    function IsVisible: Boolean;//MustReturn Visibility of item -> Result:=Self.Visible;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer);override;
    procedure Assign(Source: TPersistent);override;

    property PressState : TNVIAC_Button_Press_State read FPressState write SetPressState;
  published
    property TabOrder;
    property TabStop;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    //before win
    property Caption : TCaption read GetText write SetText;
    property Color : TColor read GetColor write SetColor;
    property Font : TFont read GetFont write SetFont;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;

    property Enabled : boolean read GetEnabled1 write SetEnabled1;

    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property ShowHint;
    property Visible : boolean read GetVisible write SetVisible;
    property Left: integer read GetLeft write SetLeft;
    property Top: integer read GetTop write SetTop;
    property Width: integer read GetWidth write SetWidth;
    property Height: integer read GetHeight write SetHeight;
    //Defined properties
    property SemiTransparence : Byte read FSemiTransparence write SetSemiTransparence default 0;
    property ButtonForm : TNVIAC_Button_Form read FButtonForm write SetButtonForm default nviacbfRect;
    property SkinImageList : TImageList read GetSkinImageList write SetSkinImageList;
    // UP,Down,Selected,Disabled
    property SkinApply : TNVIAC_Bitmap_Apply read FSkinApply write SetSkinApply default nviacbaTile;
  end;
implementation

{ TNVIAC_Button }

procedure TNVIAC_Button.Assign(Source: TPersistent);
begin
  if Source is TNVIAC_Button then
    begin
      inherited Color:=(Source as TNVIAC_Button).Color;
      inherited Font:=(Source as TNVIAC_Button).Font;
      inherited Caption:=(Source as TNVIAC_Button).Caption;

      ParentBiDiMode:=(Source as TNVIAC_Button).ParentBiDiMode;
      ParentColor:=(Source as TNVIAC_Button).ParentColor;
      ParentFont:=(Source as TNVIAC_Button).ParentFont;
      ParentShowHint:=(Source as TNVIAC_Button).ParentShowHint;
      Align:=(Source as TNVIAC_Button).Align;
      Anchors:=(Source as TNVIAC_Button).Anchors;
      BiDiMode:=(Source as TNVIAC_Button).BiDiMode;
      Constraints:=(Source as TNVIAC_Button).Constraints;
      ShowHint:=(Source as TNVIAC_Button).ShowHint;
      inherited Width:=(Source as TNVIAC_Button).Width;
      inherited Height:=(Source as TNVIAC_Button).Height;

      FSemiTransparence:=(Source as TNVIAC_Button).FSemiTransparence;
      FButtonForm:=(Source as TNVIAC_Button).FButtonForm;
      FSkinApply:=(Source as TNVIAC_Button).FSkinApply;
      FSkinImageList:=(Source as TNVIAC_Button).FSkinImageList;
    end else
    begin
      inherited Assign(Source);
    end;
end;

procedure TNVIAC_Button.ContinueBitmap;
var ARect : TRect;
    ARGN: HRGN;
    ASize : TSize;
    s : string;
    da : double;
begin
// here we continue to get the bitmap after semi...

  if assigned(AImage) and (AImage.Width>0) and (AImage.Height>0) then
    begin
      intersectrect(ARect,ACurrRect,Rect(0,0,AImage.Width,AImage.Height));
      if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,AImage.Width,AImage.Height);
      ARGN:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);
      AImage.Canvas.Brush.Color:=clYellow;
      SelectClipRGN(AImage.Canvas.Handle,ARGN);
      //Drawing

      s:=Caption;
      if (length(s)>0) and Enabled then
        begin
          AImage.Canvas.Font.Assign(Font);
          da:=2*arctan(AImage.Height/AImage.Width);
          GetTextExtentPoint32(AImage.Canvas.handle,PChar(s),length(s),aSize);
          case FButtonForm of
            nviacbfEllipRevDiag : NVIAC_RotateFont(AImage.Canvas.Font,Round(-da*1800/(2*pi)));
            nviacbfEllipDiag    : NVIAC_RotateFont(AImage.Canvas.Font,Round(da*1800/(2*pi)));
          end;

          AImage.Canvas.Brush.Style:=bsClear;
          case FButtonForm of
            nviacbfEllipRevDiag :
              begin
                TextOut(AImage.Canvas.handle,
                       (Width-Round(ASize.cx*cos(da/2)+ASize.cy*sin(da/2))) div 2+
                        Round(ASize.cy*sin(da/2)),
                       (Height-Round(ASize.cy*cos(da/2)+ASize.cx*sin(da/2))) div 2
                       ,PChar(s),length(s));
              end;
            nviacbfEllipDiag    :
              begin
                TextOut(AImage.Canvas.handle,
                       (Width-Round(ASize.cx*cos(da/2)+ASize.cy*sin(da/2))) div 2,
                       (Height+Round(ASize.cy*cos(da/2)+ASize.cx*sin(da/2))) div 2-
                       Round(ASize.cy*cos(da/2)),PChar(s),length(s));
              end;
            else
              begin
                TextOut(AImage.Canvas.handle,(Width-ASize.cx) div 2,(Height-ASize.cy) div 2,PChar(s),length(s));
              end;
          end;
          AImage.Canvas.Brush.Style:=bsSolid;
        end;

      SelectClipRGN(AImage.Canvas.Handle,0);
      DeleteObject(ARGN);
    end;
end;

constructor TNVIAC_Button.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not Is_NVIAC_BASIS_We_Got(Self,ABasis) then
    begin
      Reintroduce_NVIAC_BASIS(Self);
      Is_NVIAC_BASIS_We_Got(Self,ABasis);
    end;
  TryRegisterSemiQueque;
  ACurrRect:=Rect(0,0,0,0);
  ControlStyle:=ControlStyle+[csOpaque];
  TabStop:=True;
  inherited Font.Name:='Arial';
  inherited Font.OnChange:=FontIsChanged;
  inherited Width:=80;
  inherited Height:=20;
  inherited Caption:=Name;
  AImage:=TBitmap.Create;
  AImage.PixelFormat:=pf24Bit;
  AImage.Width:=inherited Width;
  AImage.Height:=Inherited Height;
  FSemiTransparence:=0;
  FPressState:=nviacbpsUp;
  FButtonForm:=nviacbfRect;
  FSkinImageList:=nil;
  FSkinApply:=nviacbaTile;
end;

procedure TNVIAC_Button.CreateHandle;
begin
  inherited CreateHandle;
  DefineCurrentRegion;
  SizedShowing;
end;

destructor TNVIAC_Button.Destroy;
begin
  TryUnregisterSemiQueque;
  AImage.Free;
  inherited Destroy;
end;

function TNVIAC_Button.GetAlpha: Byte;
begin
  Result:=FSemiTransparence;
end;

function TNVIAC_Button.GetAlphaBitmap: TBitmap;
begin
  Result:=nil;
end;

function TNVIAC_Button.GetColor: TColor;
begin
  Result:=Inherited Color;
end;

function TNVIAC_Button.GetSemiControl: TControl;
begin
  Result:=Self;
end;

function TNVIAC_Button.GetCurrentBitmap: TBitmap;
var ARect : TRect;
    ARGN,LRGN,DRGN,RGN1,RGN2,RGN3,TextRGN : HRGN;
    LBitmap,DBitmap,HelpBitmap : TBitmap;
    p1 : array [0..5] of TPoint;
    p2 : array [0..99] of TPoint;
    da : double;
    i,j,dx,dy : integer;
    ASize : TSize;
    s : string;
begin
  intersectrect(ARect,ACurrRect,Rect(0,0,Width,Height));
  if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,Width,Height);
  ARGN:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);

  AImage.Width:=Width;
  AImage.Height:=Height;

  SelectClipRGN(AImage.Canvas.Handle,ARGN);


  AImage.Canvas.Brush.Color:=Color;
  AImage.Canvas.FillRect(rect(0,0,AImage.Width,AImage.Height));
  if assigned(FSkinImageList) and (not (csLoading in FSkinImageList.ComponentState)) and
    (not (csReading in FSkinImageList.ComponentState)) and
    (not (csDestroying in FSkinImageList.ComponentState))then
    begin
      case FSkinApply of
        nviacbaTile :
          begin
            if Enabled then
              begin
                if Focused then
                  begin
                    case FPressState of
                      nviacbpsUp :
                        begin
                          if FSkinImageList.Count>0 then
                            begin
                              for i:=0 to AImage.Width div FSkinImageList.Width do
                                for j:=0 to AImage.Height div FSkinImageList.Height do
                                  begin
                                    FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                                      j*FSkinImageList.Height,
                                                                      0);

                                  end;
                            end;
                        end;
                      nviacbpsDown :
                        begin
                          if FSkinImageList.Count>1 then
                            begin
                              for i:=0 to AImage.Width div FSkinImageList.Width do
                                for j:=0 to AImage.Height div FSkinImageList.Height do
                                  begin
                                    FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                                      j*FSkinImageList.Height,
                                                                      1);

                                  end;
                            end else
                            begin
                              if FSkinImageList.Count>0 then
                              for i:=0 to AImage.Width div FSkinImageList.Width do
                                for j:=0 to AImage.Height div FSkinImageList.Height do
                                  begin
                                    FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                                      j*FSkinImageList.Height,
                                                                      0);

                                  end;
                            end;
                        end;
                    end;
                  end else
                  begin  //focused
                    if FSkinImageList.Count>2 then
                      begin
                        for i:=0 to AImage.Width div FSkinImageList.Width do
                          for j:=0 to AImage.Height div FSkinImageList.Height do
                            begin
                              FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                                j*FSkinImageList.Height,
                                                                2);

                            end;
                      end else
                      begin
                        if FSkinImageList.Count>0 then
                        for i:=0 to AImage.Width div FSkinImageList.Width do
                          for j:=0 to AImage.Height div FSkinImageList.Height do
                            begin
                              FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                                j*FSkinImageList.Height,
                                                                0);

                            end;
                      end;
                  end;
              end else
              begin //enabled
                if FSkinImageList.Count>3 then
                  begin
                    for i:=0 to AImage.Width div FSkinImageList.Width do
                      for j:=0 to AImage.Height div FSkinImageList.Height do
                        begin
                          FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                            j*FSkinImageList.Height,
                                                            3);

                        end;
                  end else
                  begin
                    if FSkinImageList.Count>0 then
                    for i:=0 to AImage.Width div FSkinImageList.Width do
                      for j:=0 to AImage.Height div FSkinImageList.Height do
                        begin
                          FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width,
                                                            j*FSkinImageList.Height,
                                                            0);

                        end;
                  end;
              end;
          end;
        nviacbaCenter :
          begin
            if Enabled then
              begin
                if Focused then
                  begin
                    case FPressState of
                      nviacbpsUp :
                        begin
                          if FSkinImageList.Count>0 then
                            begin
                              FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                                (AImage.Height -FSkinImageList.Height) div 2,
                                                                0);
                            end;
                        end;
                      nviacbpsDown :
                        begin
                          if FSkinImageList.Count>1 then
                            begin
                              FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                                (AImage.Height -FSkinImageList.Height) div 2,
                                                                1);
                            end else
                            begin
                              if FSkinImageList.Count>0 then
                              FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                                (AImage.Height -FSkinImageList.Height) div 2,
                                                                0);
                            end;
                        end;
                    end;
                  end else
                  begin  //focused
                    if FSkinImageList.Count>2 then
                      begin
                        FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                          (AImage.Height -FSkinImageList.Height) div 2,
                                                          2);
                      end else
                      begin
                        if FSkinImageList.Count>0 then
                        FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                          (AImage.Height -FSkinImageList.Height) div 2,
                                                          0);
                      end;
                  end;
              end else
              begin //enabled
                if FSkinImageList.Count>3 then
                  begin
                    FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                      (AImage.Height -FSkinImageList.Height) div 2,
                                                      3);
                  end else
                  begin
                    if FSkinImageList.Count>0 then
                    FSkinImageList.draw(AImage.Canvas,( AImage.Width-FSkinImageList.Width) div 2,
                                                      (AImage.Height -FSkinImageList.Height) div 2,
                                                      0);
                  end;
              end;
          end;
        nviacbaStretch :
          begin
            HelpBitmap:=TBitmap.Create;
            HelpBitmap.PixelFormat:=pf24Bit;
            HelpBitmap.Width:=FSkinImageList.Width;
            HelpBitmap.Height:=FSkinImageList.Height;
            if Enabled then
              begin
                if Focused then
                  begin
                    case FPressState of
                      nviacbpsUp :
                        begin
                           if FSkinImageList.Count>0 then  FSkinImageList.GetBitmap(0,HelpBitmap);
                        end;
                      nviacbpsDown :
                        begin
                          if FSkinImageList.Count>1 then  FSkinImageList.GetBitmap(1,HelpBitmap) else
                          if FSkinImageList.Count>0 then  FSkinImageList.GetBitmap(0,HelpBitmap);
                        end;
                    end;
                  end else
                  begin  //focused
                    if FSkinImageList.Count>2 then  FSkinImageList.GetBitmap(2,HelpBitmap) else
                    if FSkinImageList.Count>0 then  FSkinImageList.GetBitmap(0,HelpBitmap);
                  end;
              end else
              begin //enabled
                if FSkinImageList.Count>3 then  FSkinImageList.GetBitmap(3,HelpBitmap) else
                if FSkinImageList.Count>0 then  FSkinImageList.GetBitmap(0,HelpBitmap);
              end;

              AImage.Canvas.StretchDraw(Rect(0,0,AImage.Width,AImage.Height),HelpBitmap);
              HelpBitmap.Free;
          end;
        nviacbaCenterTile :
          begin
            dx:=((AImage.Width) mod FSkinImageList.Width) div 2;
            dy:=((AImage.Height) mod FSkinImageList.Height) div 2;
            if Enabled then
              begin
                if Focused then
                  begin
                    case FPressState of
                      nviacbpsUp :
                        begin
                          if FSkinImageList.Count>0 then
                            begin
                              for i:=-1 to AImage.Width div FSkinImageList.Width do
                                for j:=-1 to AImage.Height div FSkinImageList.Height do
                                  begin
                                    FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                                      j*FSkinImageList.Height+dy,
                                                                      0);

                                  end;
                            end;
                        end;
                      nviacbpsDown :
                        begin
                          if FSkinImageList.Count>1 then
                            begin
                              for i:=-1 to AImage.Width div FSkinImageList.Width do
                                for j:=-1 to AImage.Height div FSkinImageList.Height do
                                  begin
                                    FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                                      j*FSkinImageList.Height+dy,
                                                                      1);

                                  end;
                            end else
                            begin
                              if FSkinImageList.Count>0 then
                              for i:=-1 to AImage.Width div FSkinImageList.Width do
                                for j:=-1 to AImage.Height div FSkinImageList.Height do
                                  begin
                                    FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                                      j*FSkinImageList.Height+dy,
                                                                      0);

                                  end;
                            end;
                        end;
                    end;
                  end else
                  begin  //focused
                    if FSkinImageList.Count>2 then
                      begin
                        for i:=-1 to AImage.Width div FSkinImageList.Width do
                          for j:=-1 to AImage.Height div FSkinImageList.Height do
                            begin
                              FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                                j*FSkinImageList.Height+dy,
                                                                2);

                            end;
                      end else
                      begin
                        if FSkinImageList.Count>0 then
                        for i:=-1 to AImage.Width div FSkinImageList.Width do
                          for j:=-1 to AImage.Height div FSkinImageList.Height do
                            begin
                              FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                                j*FSkinImageList.Height+dy,
                                                                0);

                            end;
                      end;
                  end;
              end else
              begin //enabled
                if FSkinImageList.Count>3 then
                  begin
                    for i:=-1 to AImage.Width div FSkinImageList.Width do
                      for j:=-1 to AImage.Height div FSkinImageList.Height do
                        begin
                          FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                            j*FSkinImageList.Height+dy,
                                                            3);

                        end;
                  end else
                  begin
                    if FSkinImageList.Count>0 then
                    for i:=-1 to AImage.Width div FSkinImageList.Width do
                      for j:=-1 to AImage.Height div FSkinImageList.Height do
                        begin
                          FSkinImageList.draw(AImage.Canvas,i*FSkinImageList.Width+dx,
                                                            j*FSkinImageList.Height+dy,
                                                            0);

                        end;
                  end;
              end;
          end;
      end;
    end; //skin applying
  da:=0;
  if (Width>0) and (Height>0) then
    begin
      LRGN:=0;DRGN:=0;
      case FButtonForm of
        nviacbfOctaRect :
          begin
            if (Width>=16) and (Height>=8) then
              begin
                p2[0].x:=0;
                p2[0].y:=Height-1-4;
                p2[1].x:=0;
                p2[1].y:=4;
                p2[2].x:=8;
                p2[2].y:=0;
                p2[3].x:=Width-1-8;
                p2[3].y:=0;
                p2[4].x:=Width-1;
                p2[4].y:=4;
                p2[5].x:=Width-1-2;
                p2[5].y:=5;
                p2[6].x:=Width-1-9;
                p2[6].y:=2;
                p2[7].x:=9;
                p2[7].y:=2;
                p2[8].x:=2;
                p2[8].y:=5;
                p2[9].x:=2;
                p2[9].y:=Height-5;
                LRGN:=CreatePolygonRGN(p2,10,Alternate);
                p2[0].x:=0;
                p2[0].y:=Height-1-4;
                p2[1].x:=8;
                p2[1].y:=Height-1;
                p2[2].x:=Width-1-8;
                p2[2].y:=Height-1;
                p2[3].x:=Width-1;
                p2[3].y:=Height-1-4;
                p2[4].x:=Width-1;
                p2[4].y:=4;
                p2[5].x:=Width-1-2;
                p2[5].y:=5;
                p2[6].x:=Width-1-2;
                p2[6].y:=Height-1-5;
                p2[7].x:=Width-1-9;
                p2[7].y:=Height-1-2;
                p2[8].x:=9;
                p2[8].y:=Height-1-2;
                p2[9].x:=2;
                p2[9].y:=Height-1-5;
                DRGN:=CreatePolygonRGN(p2,10,Alternate);
                if self.Focused and Enabled then
                  begin
                      p2[0].x:=3;
                      p2[0].y:=Height-1-5;
                      p2[1].x:=3;
                      p2[1].y:=5;
                      p2[2].x:=9;
                      p2[2].y:=3;
                      p2[3].x:=Width-1-9;
                      p2[3].y:=3;
                      p2[4].x:=Width-1-3;
                      p2[4].y:=5;
                      p2[5].x:=Width-1-2;
                      p2[5].y:=5;
                      p2[6].x:=Width-1-9;
                      p2[6].y:=2;
                      p2[7].x:=9;
                      p2[7].y:=2;
                      p2[8].x:=2;
                      p2[8].y:=5;
                      p2[9].x:=2;
                      p2[9].y:=Height-5;
                      RGN1:=CreatePolygonRGN(p2,10,Alternate);
                      CombineRGN(DRGN,DRGN,RGN1,RGN_OR);
                      DeleteObject(RGN1);
                      p2[0].x:=3;
                      p2[0].y:=Height-1-5;
                      p2[1].x:=9;
                      p2[1].y:=Height-1-3;
                      p2[2].x:=Width-1-9;
                      p2[2].y:=Height-1-3;
                      p2[3].x:=Width-1-3;
                      p2[3].y:=Height-1-5;
                      p2[4].x:=Width-1-3;
                      p2[4].y:=5;
                      p2[5].x:=Width-1-2;
                      p2[5].y:=5;
                      p2[6].x:=Width-1-2;
                      p2[6].y:=Height-1-5;
                      p2[7].x:=Width-1-9;
                      p2[7].y:=Height-1-2;
                      p2[8].x:=9;
                      p2[8].y:=Height-1-2;
                      p2[9].x:=2;
                      p2[9].y:=Height-1-5;
                      RGN1:=CreatePolygonRGN(p2,10,Alternate);
                      CombineRGN(LRGN,LRGN,RGN1,RGN_OR);
                      DeleteObject(RGN1);
                  end;

            end else
            begin//width>16 height>8
              p1[0].x:=0;
              p1[0].y:=Height;
              p1[1].x:=0;
              p1[1].y:=0;
              p1[2].x:=Width;
              p1[2].y:=0;
              p1[3].x:=Width-2;
              p1[3].y:=2;
              p1[4].x:=2;
              p1[4].y:=2;
              p1[5].x:=2;
              p1[5].y:=Height-2;
              LRGN:=CreatePolygonRGN(p1,6,Alternate);
              p1[0].x:=0;
              p1[0].y:=Height;
              p1[1].x:=Width;
              p1[1].y:=Height;
              p1[2].x:=Width;
              p1[2].y:=0;
              p1[3].x:=Width-2;
              p1[3].y:=2;
              p1[4].x:=Width-2;
              p1[4].y:=Height-2;
              p1[5].x:=2;
              p1[5].y:=Height-2;
              DRGN:=CreatePolygonRGN(p1,6,Alternate);
              if self.Focused and Enabled then
                begin
                    p1[0].x:=3;
                    p1[0].y:=Height-3;
                    p1[1].x:=3;
                    p1[1].y:=3;
                    p1[2].x:=Width-3;
                    p1[2].y:=3;
                    p1[3].x:=Width-2;
                    p1[3].y:=2;
                    p1[4].x:=2;
                    p1[4].y:=2;
                    p1[5].x:=2;
                    p1[5].y:=Height-2;
                    RGN1:=CreatePolygonRGN(p1,6,Alternate);
                    CombineRGN(DRGN,DRGN,RGN1,RGN_OR);
                    DeleteObject(RGN1);
                    p1[0].x:=3;
                    p1[0].y:=Height-3;
                    p1[1].x:=Width-3;
                    p1[1].y:=Height-3;
                    p1[2].x:=Width-3;
                    p1[2].y:=3;
                    p1[3].x:=Width-2;
                    p1[3].y:=2;
                    p1[4].x:=Width-2;
                    p1[4].y:=Height-2;
                    p1[5].x:=2;
                    p1[5].y:=Height-2;
                    RGN1:=CreatePolygonRGN(p1,6,Alternate);
                    CombineRGN(LRGN,LRGN,RGN1,RGN_OR);
                    DeleteObject(RGN1);
                end;

            end;//width>8 height>4
          end;
        nviacbfEllipRevDiag :
          begin
            da:=2*arctan(Height/Width);
            for i:=0 to 99 do
              begin
                p2[i].x:=round(Width/2.0+(Width/2.0)*Sin(i*pi/50));
                p2[i].y:=round(Height/2.0+(Height/2.0)*Cos(-da/2+i*pi/50));
              end;
            LRGN:=CreatePolygonRGN(p2,100,Alternate);
            DRGN:=CreatePolygonRGN(p2,100,Alternate);
            for i:=0 to 99 do
              begin
                p2[i].x:=round(Width/2.0+(Width/2.0-2)*Sin(i*pi/50));
                p2[i].y:=round(Height/2.0+(Height/2.0-2)*Cos(-da/2+i*pi/50));
              end;
            RGN1:=CreatePolygonRGN(p2,100,Alternate);
            CombineRgn(LRGN,LRGN,RGN1,RGN_DIFF);
            CombineRgn(DRGN,DRGN,RGN1,RGN_DIFF);
            DeleteObject(RGN1);
            p1[0].x:=0;
            p1[0].y:=Height;
            p1[1].x:=Width;
            p1[1].y:=Height;
            p1[2].x:=Width;
            p1[2].y:=0;
            RGN1:=CreatePolygonRGN(p1,3,Alternate);
            CombineRgn(LRGN,LRGN,RGN1,RGN_DIFF);
            CombineRgn(DRGN,DRGN,RGN1,RGN_AND);
            DeleteObject(RGN1);
            if Enabled and Focused then
              begin
                for i:=0 to 99 do
                  begin
                    p2[i].x:=round(Width/2.0+(Width/2.0-2)*Sin(i*pi/50));
                    p2[i].y:=round(Height/2.0+(Height/2.0-2)*Cos(-da/2+i*pi/50));
                  end;
                RGN2:=CreatePolygonRGN(p2,100,Alternate);
                RGN3:=CreatePolygonRGN(p2,100,Alternate);
                for i:=0 to 99 do
                  begin
                    p2[i].x:=round(Width/2.0+(Width/2.0-3)*Sin(i*pi/50));
                    p2[i].y:=round(Height/2.0+(Height/2.0-3)*Cos(-da/2+i*pi/50));
                  end;
                RGN1:=CreatePolygonRGN(p2,100,Alternate);
                CombineRgn(RGN2,RGN2,RGN1,RGN_DIFF);
                CombineRgn(RGN3,RGN3,RGN1,RGN_DIFF);
                DeleteObject(RGN1);
                p1[0].x:=0;
                p1[0].y:=Height;
                p1[1].x:=Width;
                p1[1].y:=Height;
                p1[2].x:=Width;
                p1[2].y:=0;
                RGN1:=CreatePolygonRGN(p1,3,Alternate);
                CombineRgn(RGN2,RGN2,RGN1,RGN_DIFF);
                CombineRgn(RGN3,RGN3,RGN1,RGN_AND);
                DeleteObject(RGN1);
                CombineRgn(LRGN,LRGN,RGN3,RGN_OR);
                CombineRgn(DRGN,DRGN,RGN2,RGN_OR);
                DeleteObject(RGN2);
                DeleteObject(RGN3);
              end;
          end;
        nviacbfEllipDiag :
          begin
            da:=2*arctan(Height/Width);
            for i:=0 to 99 do
              begin
                p2[i].x:=round(Width/2.0+(Width/2.0)*Sin(i*pi/50));
                p2[i].y:=round(Height/2.0+(Height/2.0)*Cos(da/2+i*pi/50));
              end;
            LRGN:=CreatePolygonRGN(p2,100,Alternate);
            DRGN:=CreatePolygonRGN(p2,100,Alternate);
            for i:=0 to 99 do
              begin
                p2[i].x:=round(Width/2.0+(Width/2.0-2)*Sin(i*pi/50));
                p2[i].y:=round(Height/2.0+(Height/2.0-2)*Cos(da/2+i*pi/50));
              end;
            RGN1:=CreatePolygonRGN(p2,100,Alternate);
            CombineRgn(LRGN,LRGN,RGN1,RGN_DIFF);
            CombineRgn(DRGN,DRGN,RGN1,RGN_DIFF);
            DeleteObject(RGN1);
            p1[0].x:=0;
            p1[0].y:=Height;
            p1[1].x:=Width;
            p1[1].y:=Height;
            p1[2].x:=Width;
            p1[2].y:=0;
            RGN1:=CreatePolygonRGN(p1,3,Alternate);
            CombineRgn(LRGN,LRGN,RGN1,RGN_DIFF);
            CombineRgn(DRGN,DRGN,RGN1,RGN_AND);
            DeleteObject(RGN1);
            if Enabled and Focused then
              begin
                for i:=0 to 99 do
                  begin
                    p2[i].x:=round(Width/2.0+(Width/2.0-2)*Sin(i*pi/50));
                    p2[i].y:=round(Height/2.0+(Height/2.0-2)*Cos(da/2+i*pi/50));
                  end;
                RGN2:=CreatePolygonRGN(p2,100,Alternate);
                RGN3:=CreatePolygonRGN(p2,100,Alternate);
                for i:=0 to 99 do
                  begin
                    p2[i].x:=round(Width/2.0+(Width/2.0-3)*Sin(i*pi/50));
                    p2[i].y:=round(Height/2.0+(Height/2.0-3)*Cos(da/2+i*pi/50));
                  end;
                RGN1:=CreatePolygonRGN(p2,100,Alternate);
                CombineRgn(RGN2,RGN2,RGN1,RGN_DIFF);
                CombineRgn(RGN3,RGN3,RGN1,RGN_DIFF);
                DeleteObject(RGN1);
                p1[0].x:=0;
                p1[0].y:=Height;
                p1[1].x:=Width;
                p1[1].y:=Height;
                p1[2].x:=Width;
                p1[2].y:=0;
                RGN1:=CreatePolygonRGN(p1,3,Alternate);
                CombineRgn(RGN2,RGN2,RGN1,RGN_DIFF);
                CombineRgn(RGN3,RGN3,RGN1,RGN_AND);
                DeleteObject(RGN1);
                CombineRgn(LRGN,LRGN,RGN3,RGN_OR);
                CombineRgn(DRGN,DRGN,RGN2,RGN_OR);
                DeleteObject(RGN2);
                DeleteObject(RGN3);
              end;
          end;
        nviacbfElliptic :
          begin
            LRGN:=CreateEllipticRGN(0,0,Width,Height);
            RGN1:=CreateEllipticRGN(2,2,Width-2,Height-2);
            CombineRgn(LRGN,LRGN,RGN1,RGN_DIFF);
            DRGN:=CreateEllipticRGN(0,0,Width,Height);
            CombineRgn(DRGN,DRGN,RGN1,RGN_DIFF);
            DeleteObject(RGN1);
            p1[0].x:=0;
            p1[0].y:=Height;
            p1[1].x:=Width;
            p1[1].y:=Height;
            p1[2].x:=Width;
            p1[2].y:=0;
            RGN1:=CreatePolygonRGN(p1,3,Alternate);
            CombineRgn(LRGN,LRGN,RGN1,RGN_DIFF);
            CombineRgn(DRGN,DRGN,RGN1,RGN_AND);
            DeleteObject(RGN1);
            if Enabled and Focused then
              begin
                RGN2:=CreateEllipticRGN(2,2,Width-2,Height-2);
                RGN1:=CreateEllipticRGN(3,3,Width-3,Height-3);
                CombineRgn(RGN2,RGN2,RGN1,RGN_DIFF);
                RGN3:=CreateEllipticRGN(2,2,Width-2,Height-2);
                CombineRgn(RGN3,RGN3,RGN1,RGN_DIFF);
                DeleteObject(RGN1);
                p1[0].x:=0;
                p1[0].y:=Height;
                p1[1].x:=Width;
                p1[1].y:=Height;
                p1[2].x:=Width;
                p1[2].y:=0;
                RGN1:=CreatePolygonRGN(p1,3,Alternate);
                CombineRgn(RGN2,RGN2,RGN1,RGN_DIFF);
                CombineRgn(RGN3,RGN3,RGN1,RGN_AND);
                DeleteObject(RGN1);
                CombineRgn(LRGN,LRGN,RGN3,RGN_OR);
                CombineRgn(DRGN,DRGN,RGN2,RGN_OR);
                DeleteObject(RGN2);
                DeleteObject(RGN3);
              end;
          end;
        nviacbfRect :
          begin
            p1[0].x:=0;
            p1[0].y:=Height;
            p1[1].x:=0;
            p1[1].y:=0;
            p1[2].x:=Width;
            p1[2].y:=0;
            p1[3].x:=Width-2;
            p1[3].y:=2;
            p1[4].x:=2;
            p1[4].y:=2;
            p1[5].x:=2;
            p1[5].y:=Height-2;
            LRGN:=CreatePolygonRGN(p1,6,Alternate);
            p1[0].x:=0;
            p1[0].y:=Height;
            p1[1].x:=Width;
            p1[1].y:=Height;
            p1[2].x:=Width;
            p1[2].y:=0;
            p1[3].x:=Width-2;
            p1[3].y:=2;
            p1[4].x:=Width-2;
            p1[4].y:=Height-2;
            p1[5].x:=2;
            p1[5].y:=Height-2;
            DRGN:=CreatePolygonRGN(p1,6,Alternate);
            if self.Focused and Enabled then
              begin
                  p1[0].x:=3;
                  p1[0].y:=Height-3;
                  p1[1].x:=3;
                  p1[1].y:=3;
                  p1[2].x:=Width-3;
                  p1[2].y:=3;
                  p1[3].x:=Width-2;
                  p1[3].y:=2;
                  p1[4].x:=2;
                  p1[4].y:=2;
                  p1[5].x:=2;
                  p1[5].y:=Height-2;
                  RGN1:=CreatePolygonRGN(p1,6,Alternate);
                  CombineRGN(DRGN,DRGN,RGN1,RGN_OR);
                  DeleteObject(RGN1);
                  p1[0].x:=3;
                  p1[0].y:=Height-3;
                  p1[1].x:=Width-3;
                  p1[1].y:=Height-3;
                  p1[2].x:=Width-3;
                  p1[2].y:=3;
                  p1[3].x:=Width-2;
                  p1[3].y:=2;
                  p1[4].x:=Width-2;
                  p1[4].y:=Height-2;
                  p1[5].x:=2;
                  p1[5].y:=Height-2;
                  RGN1:=CreatePolygonRGN(p1,6,Alternate);
                  CombineRGN(LRGN,LRGN,RGN1,RGN_OR);
                  DeleteObject(RGN1);
              end;
          end;//Rect;
      end;
      //Creating of Text RGN
      AImage.Canvas.Font.Assign(Font);

      s:=Caption;
      if length(s)>0 then
        begin
          GetTextExtentPoint32(AImage.Canvas.handle,PChar(s),length(s),aSize);
          case FButtonForm of
            nviacbfEllipRevDiag : NVIAC_RotateFont(AImage.Canvas.Font,Round(-da*1800/(2*pi)));
            nviacbfEllipDiag    : NVIAC_RotateFont(AImage.Canvas.Font,Round(da*1800/(2*pi)));
          end;
          BeginPath(AImage.Canvas.handle);
          AImage.Canvas.Brush.Style:=bsClear;
          case FButtonForm of
            nviacbfEllipRevDiag :
              begin
                TextOut(AImage.Canvas.handle,
                       (Width-Round(ASize.cx*cos(da/2)+ASize.cy*sin(da/2))) div 2+1+
                        Round(ASize.cy*sin(da/2)),
                       (Height-Round(ASize.cy*cos(da/2)+ASize.cx*sin(da/2))) div 2+1
                       ,PChar(s),length(s));
              end;
            nviacbfEllipDiag    :
              begin
                TextOut(AImage.Canvas.handle,
                       (Width-Round(ASize.cx*cos(da/2)+ASize.cy*sin(da/2))) div 2+1,
                       (Height+Round(ASize.cy*cos(da/2)+ASize.cx*sin(da/2))) div 2+1-
                       Round(ASize.cy*cos(da/2)),PChar(s),length(s));
              end;
            else
              begin
                TextOut(AImage.Canvas.handle,(Width-ASize.cx) div 2+1,(Height-ASize.cy) div 2+1,PChar(s),length(s));
              end;
          end;

          AImage.Canvas.Brush.Style:=bsSolid;
          EndPath(AImage.Canvas.handle);
          TextRGN:=PathToRegion(AImage.Canvas.handle);
          CombineRGN(DRGN,DRGN,TextRGN,RGN_OR);
          DeleteObject(TextRGN);
          BeginPath(AImage.Canvas.handle);
          AImage.Canvas.Brush.Style:=bsClear;
          case FButtonForm of
            nviacbfEllipRevDiag :
              begin
                TextOut(AImage.Canvas.handle,
                       (Width-Round(ASize.cx*cos(da/2)+ASize.cy*sin(da/2))) div 2-1+
                        Round(ASize.cy*sin(da/2)),
                       (Height-Round(ASize.cy*cos(da/2)+ASize.cx*sin(da/2))) div 2-1
                       ,PChar(s),length(s));
              end;
            nviacbfEllipDiag    :
              begin
                TextOut(AImage.Canvas.handle,
                       (Width-Round(ASize.cx*cos(da/2)+ASize.cy*sin(da/2))) div 2-1,
                       (Height+Round(ASize.cy*cos(da/2)+ASize.cx*sin(da/2))) div 2-1-
                       Round(ASize.cy*cos(da/2)),PChar(s),length(s));
              end;
            else
              begin
                TextOut(AImage.Canvas.handle,(Width-ASize.cx) div 2-1,(Height-ASize.cy) div 2-1,PChar(s),length(s));
              end;
          end;
          AImage.Canvas.Brush.Style:=bsSolid;
          EndPath(AImage.Canvas.handle);
          TextRGN:=PathToRegion(AImage.Canvas.handle);
          CombineRGN(LRGN,LRGN,TextRGN,RGN_OR);
          DeleteObject(TextRGN);
        end;

      LBitmap:=NVIAC_GetLBitmapASM(AImage,32);
      DBitmap:=NVIAC_GetDBitmapASM(AImage,32);
      SelectClipRGN(AImage.Canvas.Handle,0);
      CombineRGN(LRGN,LRGN,ARGN,RGN_AND);

      CombineRGN(DRGN,DRGN,ARGN,RGN_AND);
      case FPressState of
        nviacbpsUp :
          begin
            SelectClipRGN(AImage.Canvas.Handle,LRGN);
            AImage.Canvas.Draw(0,0,LBitmap);
            SelectClipRGN(AImage.Canvas.Handle,DRGN);
            AImage.Canvas.Draw(0,0,DBitmap);
          end;
        nviacbpsDown :
          begin
            if Enabled then
              begin
                SelectClipRGN(AImage.Canvas.Handle,DRGN);
                AImage.Canvas.Draw(0,0,LBitmap);
                SelectClipRGN(AImage.Canvas.Handle,LRGN);
                AImage.Canvas.Draw(0,0,DBitmap);
              end else
              begin
                SelectClipRGN(AImage.Canvas.Handle,LRGN);
                AImage.Canvas.Draw(0,0,LBitmap);
                SelectClipRGN(AImage.Canvas.Handle,DRGN);
                AImage.Canvas.Draw(0,0,DBitmap);
              end;
          end;
      end;
      LBitmap.Free;
      DBitmap.Free;
      SelectClipRGN(AImage.Canvas.Handle,0);


      DeleteObject(LRGN);
      DeleteObject(DRGN);
    end else
    begin
      SelectClipRGN(AImage.Canvas.Handle,0);
    end;  
  DeleteObject(ARGN);
  result:=AImage;
end;

function TNVIAC_Button.GetCurrentOffset: TPoint;
var APos1 : TSize;
begin
  APos1:=GetControlPosition(Self);
  Result.x:=APos1.cx;
  Result.y:=APos1.cy;
end;

function TNVIAC_Button.GetCurrentRegion: HRGN;
var ARect : TRect;
begin
  intersectrect(ARect,ACurrRect,Rect(0,0,AImage.Width,AImage.Height));
  if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,AImage.Width,AImage.Height);
  Result:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);
end;

function TNVIAC_Button.GetCurrentSize: TSize;
begin
  Result.cx:=AImage.Width;
  Result.cy:=AImage.Height;
end;

function TNVIAC_Button.GetEnabled1: boolean;
begin
  Result:=inherited Enabled;
end;

function TNVIAC_Button.GetFont: TFont;
begin
  Result:=inherited Font;
end;

function TNVIAC_Button.GetFullBitmap: TBitmap;
begin
  Result:=AImage;
end;

function TNVIAC_Button.GetHeight: integer;
begin
  Result:=inherited Height;
end;

function TNVIAC_Button.GetLeft: integer;
begin
  Result:=inherited Left;
end;

function TNVIAC_Button.GetSemiClass: Byte;
begin
  Result:=1;
end;

function TNVIAC_Button.GetText: TCaption;
begin
  Result:=inherited Caption;
end;

function TNVIAC_Button.GetTop: integer;
begin
  Result:=inherited Top;
end;

function TNVIAC_Button.GetVisible: boolean;
begin
  Result:=inherited Visible;
end;

function TNVIAC_Button.GetWidth: integer;
begin
  Result:=inherited Width;
end;

function TNVIAC_Button.IsVisible: Boolean;
begin
  Result:=Visible;
end;

procedure TNVIAC_Button.Paint;
begin
  if Assigned(AImage) and HandleAllocated then
    begin
      Canvas.Draw(0,0,AImage);
    end;
end;

procedure TNVIAC_Button.ReintroduceBasis(ANewBasis: TNVIAC_BASIS);
begin
  ABasis:=ANewBasis;
end;

procedure TNVIAC_Button.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  SizedHiding;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  DefineCurrentRegion;
  SizedShowing;
end;

procedure TNVIAC_Button.SetColor(Value: TColor);
begin
  inherited Color:=Value;
  SizedShowing;
end;

procedure TNVIAC_Button.SetCurrRect(ARect: TRect);
begin
  ACurrRect.Left:=ARect.Left;
  ACurrRect.Top:=ARect.Top;
  ACurrRect.Right:=ARect.Right;
  ACurrRect.Bottom:=ARect.Bottom;
end;

procedure TNVIAC_Button.SetEnabled1(Value: boolean);
begin
  inherited Enabled:=Value;
  SizedShowing;
end;

procedure TNVIAC_Button.SetFont(Value: TFont);
begin
  inherited Font:=Value;
  SizedShowing;
end;

procedure TNVIAC_Button.SetHeight(Value: integer);
begin
  inherited Height:=Value;
end;

procedure TNVIAC_Button.SetLeft(Value: integer);
begin
  inherited Left:=Value;
end;

procedure TNVIAC_Button.SetText(Value: TCaption);
begin
  inherited Caption:=Value;
  SizedShowing;
end;

procedure TNVIAC_Button.SetTop(Value: integer);
begin
  inherited Top:=Value;
end;

procedure TNVIAC_Button.SetVisible(Value: boolean);
begin
  if Value<>(inherited Visible) then
    begin
      inherited Visible:=Value;
      DefineCurrentRegion;
      if Value then
        begin
          SizedShowing;
        end else
        begin
          SizedHiding;
        end;
    end;
end;

procedure TNVIAC_Button.SetWidth(Value: integer);
begin
  inherited Width:=Value;
end;

procedure TNVIAC_Button.TryPaint;
begin
  if HandleAllocated then Repaint;//Invalidate;//
end;

procedure TNVIAC_Button.TryRegisterSemiQueque;
begin
  if assigned(ABasis) then ABasis.AddToSemiQueue(self);
end;

procedure TNVIAC_Button.TryUnregisterSemiQueque;
begin
  if assigned(ABasis) then ABasis.RemoveFromSemiQueue(self);
end;
procedure TNVIAC_Button.SizedHiding;
begin
  if assigned(ABasis) and Assigned(AImage) and assigned(Parent) and parent.handleallocated and HandleAllocated then
  begin
    AImage.Width:=Width;
    AImage.Height:=Height;
    ABasis.SemiIsHiding(self);
  end;
end;
procedure TNVIAC_Button.SizedShowing;
begin
  if HandleAllocated then
    begin
      GetBaseBitmap;
    end;
end;
procedure TNVIAC_Button.FontIsChanged(Sender: TObject);
begin
  SizedShowing;
end;
procedure TNVIAC_Button.GetBaseBitmap;
begin
 if assigned(AImage) and (inherited Width>0) and (inherited Height>0) and HandleAllocated then
    begin
      AImage.Width:=inherited Width;
      AImage.Height:=inherited Height;
      if Assigned(ABasis) then ABasis.SemiIsUpdated(self);
    end;
end;
procedure TNVIAC_Button.SetSemiTransparence(Value: Byte);
begin
  if Value<>FSemiTransparence then
    begin
      FSemiTransparence := Value;
      SizedShowing;
    end;
end;

procedure TNVIAC_Button.SetPressState(Value: TNVIAC_Button_Press_State);
begin
  if Value<>FPressState then
    begin
      FPressState := Value;
      SizedShowing;
    end;
end;

procedure TNVIAC_Button.WMKEYDOWN(var message: TMessage);
begin
  inherited;
  case message.WParam of
    VK_SPACE,VK_RETURN :
      begin
        FMouseDown:=True;
        SetFocus;
        PressState:=nviacbpsDown;
      end;
  end;
end;

procedure TNVIAC_Button.WMKEYUP(var message: TMessage);
begin
  inherited;
  case message.WParam of
    VK_SPACE,VK_RETURN :
      begin
        FMouseDown:=False;
        PressState:=nviacbpsUp;
        if assigned(OnClick) then OnClick(Self);
      end;
  end;
end;

procedure TNVIAC_Button.WMLBUTTONDOWN(var message: TMessage);
begin
  inherited;
  FMouseDown:=True;
  self.SetFocus;
  PressState:=nviacbpsDown;
end;

procedure TNVIAC_Button.WMLBUTTONUP(var message: TMessage);
begin
  inherited;
  FMouseDown:=False;
  PressState:=nviacbpsUp;
end;

procedure TNVIAC_Button.DefineCurrentRegion;
var RGN : HRGN;
    p1 : array [0..99] of TPoint;
    i : integer;
    da : double;
begin
  if handleallocated then
    begin
      if Visible then
        begin
          case FButtonForm of
            nviacbfOctaRect :
              begin
                if (Width>=16) and (Height>=8) then
                  begin
                    p1[0].x:=0;
                    p1[0].y:=4;
                    p1[1].x:=8;
                    p1[1].y:=0;
                    p1[2].x:=Width-1-8;
                    p1[2].y:=0;
                    p1[3].x:=Width-1;
                    p1[3].y:=4;
                    p1[4].x:=Width-1;
                    p1[4].y:=Height-1-4;
                    p1[5].x:=Width-1-8;
                    p1[5].y:=Height-1;
                    p1[6].x:=8;
                    p1[6].y:=Height-1;
                    p1[7].x:=0;
                    p1[7].y:=Height-1-4;
                    RGN:=CreatePolygonRGN(p1,8,Alternate);
                    SetWindowRGN(Handle,RGN,TRUE);
                  end else
                  begin
                    RGN:=CreateRectRGN(0,0,Width,Height);
                    SetWindowRGN(Handle,RGN,TRUE);
                  end;
              end;
            nviacbfRect :
              begin
                SetWindowRGN(Handle,0,TRUE);
              end;
            nviacbfElliptic :
              begin
                RGN:=CreateEllipticRGN(0,0,Width,Height);
                SetWindowRGN(Handle,RGN,TRUE);
              end;
            nviacbfEllipDiag :
              begin
                if (Width>0) and (Height>0) then
                  begin
                    da:=2*arctan(Height/Width);
                    for i:=0 to 99 do
                      begin
                        p1[i].x:=round(Width/2.0+(Width/2.0)*Sin(i*pi/50));
                        p1[i].y:=round(Height/2.0+(Height/2.0)*Cos(+da/2+i*pi/50));
                      end;
                    RGN:=CreatePolygonRGN(p1,100,Alternate);
                    SetWindowRGN(Handle,RGN,TRUE); 
                  end else
                  begin
                    RGN:=CreateEllipticRGN(0,0,Width,Height);
                    SetWindowRGN(Handle,RGN,TRUE);
                  end;
              end;
            nviacbfEllipRevDiag :
              begin
                if (Width>0) and (Height>0) then
                  begin
                    da:=2*arctan(Height/Width);
                    for i:=0 to 99 do
                      begin
                        p1[i].x:=round(Width/2.0+(Width/2.0)*Sin(i*pi/50));
                        p1[i].y:=round(Height/2.0+(Height/2.0)*Cos(-da/2+i*pi/50));
                      end;
                    RGN:=CreatePolygonRGN(p1,100,Alternate);
                    SetWindowRGN(Handle,RGN,TRUE); 
                  end else
                  begin
                    RGN:=CreateEllipticRGN(0,0,Width,Height);
                    SetWindowRGN(Handle,RGN,TRUE);
                  end;
              end;  
          end;
        end else
        begin
          RGN:=CreateRectRGN(0,0,0,0);
          SetWindowRGN(Handle,RGN,TRUE);
        end;
    end;
end;

procedure TNVIAC_Button.SetButtonForm(Value: TNVIAC_Button_Form);
begin
  if Value<>FButtonForm then
    begin
      SizedHiding;
      FButtonForm := Value;
      DefineCurrentRegion;
      SizedShowing;
    end;
end;

procedure TNVIAC_Button.WMLBUTTONDBLCLK(var message: TMessage);
begin
  perform(WM_LBUTTONDOWN,0,0); 
end;

procedure TNVIAC_Button.WMKILLFOCUS(var message: TMessage);
begin
  inherited;
  SizedShowing;
end;

procedure TNVIAC_Button.WMSETFOCUS(var message: TMessage);
begin
  inherited;
  if not FMouseDown then SizedShowing;
end;

procedure TNVIAC_Button.SetSkinApply(Value: TNVIAC_Bitmap_Apply);
begin
  if Value<>FSkinApply then
    begin
      FSkinApply := Value;
      if assigned(FSkinImageList) then SizedShowing;
    end;
end;

procedure TNVIAC_Button.SetSkinImageList(Value: TImageList);
begin
  FSkinImageList := Value;
  SizedShowing;
end;

function TNVIAC_Button.GetSkinImageList: TImageList;
begin
  if assigned(FSkinImageList) then
    begin
      Result:=FSkinImageList;
    end else
    begin
      FSkinImageList:=nil;
      Result:=FSkinImageList;
    end;
end;





end.
