unit NVIAC_Image_Unit;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,NVIAC_BASIS,nviac_textlines_unit,
  nviac_bitmap_unit,ExtCtrls;
Type
TNVIAC_Image = class(TGraphicControl,INVIAC_SEMITRANSPARENCE)
private
  ABasis : TNVIAC_BASIS;
  ACurrRect : TRect;
  AImage : TBitmap;
  AAlphaImage : TBitmap;
  FSemiTransparence: Byte;
  FPicture: TPicture;
  FOnProgress: TProgressEvent;
  FStretch: Boolean;
  FCenter: Boolean;
  FIncrementalDisplay: Boolean;
  FDrawing: Boolean;
    FTransparent: Boolean;
    FTranspRegPower: Byte;
  function GetAutoSize: Boolean;
  function GetEnabled1: boolean;
  function GetVisible: boolean;
  procedure SetAutoSize(Value: Boolean);
  procedure SetEnabled1(Value: boolean);
  procedure SetSemiTransparence(Value: Byte);
  procedure SetVisible(Value: boolean);
  procedure GetBaseBitmap;
  function GetCanvas: TCanvas;
  procedure PictureChanged(Sender: TObject);
  procedure SetCenter(Value: Boolean);
  procedure SetPicture(Value: TPicture);
  procedure SetStretch(Value: Boolean);
  function GetHeight: integer;
  function GetLeft: integer;
  function GetTop: integer;
  function GetWidth: integer;
  procedure SetHeight(Value: integer);
  procedure SetLeft(Value: integer);
  procedure SetTop(Value: integer);
  procedure SetWidth(Value: integer);
  function GetColor: TColor;
  procedure SetColor(Value: TColor);
  procedure SetTransparent(Value: Boolean);
    procedure SetTranspRegPower(Value: Byte);
protected
  procedure Paint; override;
  procedure SizedHiding;
  procedure SizedShowing;
  function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
  function DestRect: TRect;
  function DoPaletteChange: Boolean;
  function GetPalette: HPALETTE; override;
  procedure Progress(Sender: TObject; Stage: TProgressStage;
                     PercentDone: Byte; RedrawNow: Boolean; const R: TRect; const Msg: string); dynamic;
  procedure Loaded; override;    
public
  procedure ReintroduceBasis(ANewBasis : TNVIAC_BASIS);
  procedure TryRegisterSemiQueque;
  procedure TryUnregisterSemiQueque;
  function GetFullBitmap: TBitmap;
  function GetCurrentSize : TSize;
  function GetCurrentBitmap: TBitmap;
  function GetCurrentOffset: TPoint;
  function GetCurrentRegion: HRGN;
  function GetSemiClass : Byte;//1 - Alpha,2 - bitmap
  function GetAlpha : Byte;
  function GetAlphaBitmap : TBitmap;
  procedure ContinueBitmap;
  procedure TryPaint;
  procedure SetCurrRect(ARect : TRect);
  function GetSemiControl: TControl;
  function IsVisible: Boolean;

  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer);override;
  property Canvas: TCanvas read GetCanvas;
published
  property AutoSize : Boolean read GetAutoSize write SetAutoSize;
  property Enabled : boolean read GetEnabled1 write SetEnabled1;
  property Visible : boolean read GetVisible write SetVisible;

  property Align;
  property Anchors;
  property Center: Boolean read FCenter write SetCenter default False;
  property Constraints;
  property DragCursor;
  property DragKind;
  property DragMode;
  property IncrementalDisplay: Boolean read FIncrementalDisplay write FIncrementalDisplay default False;
  property ParentShowHint;
  property Picture: TPicture read FPicture write SetPicture;
  property PopupMenu;
  property ShowHint;
  property Stretch: Boolean read FStretch write SetStretch default False;
  property OnClick;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDock;
  property OnEndDrag;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnProgress: TProgressEvent read FOnProgress write FOnProgress;
  property OnStartDock;
  property OnStartDrag;
  property Left: integer read GetLeft write SetLeft;
  property Top: integer read GetTop write SetTop;
  property Width: integer read GetWidth write SetWidth;
  property Height: integer read GetHeight write SetHeight;
  property Color : TColor read GetColor write SetColor;
  property ParentColor;

  property SemiTransparence : Byte read FSemiTransparence write SetSemiTransparence default 0;
  property Transparent : Boolean read FTransparent write SetTransparent;
  property TranspRegPower : Byte read FTranspRegPower write SetTranspRegPower default 255;
end;
implementation

{ TNVIAC_IMAGE }


function TNVIAC_Image.CanAutoSize(var NewWidth,NewHeight: Integer): Boolean;
begin
  Result := True;
  if not (csDesigning in ComponentState) or (Picture.Width > 0) and
    (Picture.Height > 0) then
  begin
    if Align in [alNone, alLeft, alRight] then
      NewWidth := Picture.Width;
    if Align in [alNone, alTop, alBottom] then
      NewHeight := Picture.Height;
  end;
end;

procedure TNVIAC_IMAGE.ContinueBitmap;
var ARect : TRect;
    ARGN: HRGN;
begin
// here we continue to get the bitmap after semi...
  if assigned(AImage) and (AImage.Width>0) and (AImage.Height>0) then
    begin
      intersectrect(ARect,ACurrRect,Rect(0,0,AImage.Width,AImage.Height));
      if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,AImage.Width,AImage.Height);
      ARGN:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);
      SelectClipRGN(AImage.Canvas.Handle,ARGN);

      SelectClipRGN(AImage.Canvas.Handle,0);
      DeleteObject(ARGN);
    end;
end;

constructor TNVIAC_IMAGE.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not Is_NVIAC_BASIS_We_Got(Self,ABasis) then
    begin
      Reintroduce_NVIAC_BASIS(Self);
      Is_NVIAC_BASIS_We_Got(Self,ABasis);
    end;
  TryRegisterSemiQueque;
  ControlStyle := ControlStyle + [csReplicatable]+[csOpaque];
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FPicture.OnProgress := Progress;
  inherited SetBounds(Left,Top,105,105);
  AImage:=TBitmap.Create;
  AImage.PixelFormat:=pf24Bit;
  AImage.Width:=Width;
  AImage.Height:=Height;
  FSemiTransparence:=0;
  Picture.OnChange := PictureChanged;
  ACurrRect:=Rect(0,0,0,0);
  FTransparent:=False;
  FTranspRegPower:=255;
  AAlphaImage:=nil;
end;

function TNVIAC_Image.DestRect: TRect;
begin
  if Stretch then
    Result := ClientRect
  else if Center then
    Result := Bounds((Width - Picture.Width) div 2, (Height - Picture.Height) div 2,
      Picture.Width, Picture.Height)
  else
    Result := Rect(0, 0, Picture.Width, Picture.Height);
end;

destructor TNVIAC_IMAGE.Destroy;
begin
  TryUnregisterSemiQueque;
  if Assigned(AAlphaImage) then AAlphaImage.Free;
  AImage.Free;
  FPicture.Free;
  inherited Destroy;
end;

function TNVIAC_Image.DoPaletteChange: Boolean;
var
  ParentForm: TCustomForm;
  Tmp: TGraphic;
begin
  Result := False;
  Tmp := Picture.Graphic;
  if Visible and (not (csLoading in ComponentState)) and (Tmp <> nil) and
    (Tmp.PaletteModified) then
  begin
    if (Tmp.Palette = 0) then
      Tmp.PaletteModified := False
    else
    begin
      ParentForm := GetParentForm(Self);
      if Assigned(ParentForm) and ParentForm.Active and Parentform.HandleAllocated then
      begin
        if FDrawing then
          ParentForm.Perform(wm_QueryNewPalette, 0, 0)
        else
          PostMessage(ParentForm.Handle, wm_QueryNewPalette, 0, 0);
        Result := True;
        Tmp.PaletteModified := False;
      end;
    end;
  end;
  if Visible then SizedShowing;
end;

function TNVIAC_IMAGE.GetAlpha: Byte;
begin
  Result:=FSemiTransparence;
end;

function TNVIAC_IMAGE.GetAlphaBitmap: TBitmap;
var LBit : TBitmap;
begin
  if not assigned(AAlphaImage) then
    begin
      AAlphaImage:=TBitmap.Create;
      AAlphaImage.PixelFormat:=pf24Bit;
    end;
  AAlphaImage.Width:=AImage.Width;
  AAlphaImage.Height:=AImage.Height;
  if (AImage.Width>0) and (AImage.Height>0) then
    begin
      AAlphaImage.Canvas.Brush.Color:=RGB(FTranspRegPower,FTranspRegPower,FTranspRegPower);
      AAlphaImage.Canvas.FillRect(Rect(0,0, AAlphaImage.Width, AAlphaImage.Height));
      LBit:=NVIAC_GetTransparentBitmapMask(Picture.Bitmap,Picture.Bitmap.Canvas.Pixels[0,0],FTranspRegPower,FSemiTransparence);
      AAlphaImage.Canvas.StretchDraw(DestRect,LBit);
      LBit.Free;
    end;
  Result:=AAlphaImage;
end;

function TNVIAC_IMAGE.GetAutoSize: Boolean;
begin
  Result:=inherited AutoSize;
end;

procedure TNVIAC_IMAGE.GetBaseBitmap;
begin
  if assigned(AImage) and (inherited Width>0) and (inherited Height>0)then
    begin
      AImage.Width:=inherited Width;
      AImage.Height:=inherited Height;
      if Assigned(ABasis) then ABasis.SemiIsUpdated(self);
    end;
end;


function TNVIAC_Image.GetCanvas: TCanvas;
begin
  Result:=inherited Canvas;
end;

function TNVIAC_Image.GetColor: TColor;
begin
  Result:=inherited Color;
end;

function TNVIAC_IMAGE.GetCurrentBitmap: TBitmap;
var ARect : TRect;
    ARGN: HRGN;
    Save: Boolean;
begin
  intersectrect(ARect,ACurrRect,Rect(0,0,Width,Height));
  if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,Width,Height);
  ARGN:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);

  AImage.Width:=Width;
  AImage.Height:=Height;
  SelectClipRGN(AImage.Canvas.Handle,ARGN);
  AImage.Canvas.Brush.Style := bsSolid;
  AImage.Canvas.Brush.Color:=Color;
  AImage.Canvas.FillRect(Rect(0,0,AImage.Width,AImage.Height));

  if csDesigning in ComponentState then
    begin
      AImage.Canvas.Pen.Style := psDash;
      AImage.Canvas.Brush.Style := bsClear;
      AImage.Canvas.Rectangle(0, 0, Width, Height);
    end;
  Save := FDrawing;
  FDrawing := True;
  try
    AImage.Canvas.StretchDraw(DestRect, Picture.Graphic);
  finally
    FDrawing := Save;
  end;

  SelectClipRGN(AImage.Canvas.Handle,0);
  DeleteObject(ARGN);
  result:=AImage;
end;

function TNVIAC_IMAGE.GetCurrentOffset: TPoint;
var APos1 : TSize;
begin
  APos1:=GetControlPosition(Self);
  Result.x:=APos1.cx;
  Result.y:=APos1.cy;
end;

function TNVIAC_IMAGE.GetCurrentRegion: HRGN;
var ARect : TRect;
begin
  intersectrect(ARect,ACurrRect,Rect(0,0,AImage.Width,AImage.Height));
  if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,AImage.Width,AImage.Height);
  Result:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);
end;

function TNVIAC_IMAGE.GetCurrentSize: TSize;
begin
  Result.cx:=AImage.Width;
  Result.cy:=AImage.Height;
end;

function TNVIAC_IMAGE.GetEnabled1: boolean;
begin
  Result:=inherited Enabled;
end;

function TNVIAC_IMAGE.GetFullBitmap: TBitmap;
begin
  Result:=AImage;
end;


function TNVIAC_Image.GetHeight: integer;
begin
  Result:=inherited Height;
end;

function TNVIAC_Image.GetLeft: integer;
begin
  Result:=inherited Left;
end;

function TNVIAC_Image.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture.Graphic <> nil then
    Result := FPicture.Graphic.Palette;
end;

function TNVIAC_IMAGE.GetSemiClass: Byte;
begin
  if FTransparent then Result:=2 else Result:=1;
end;

function TNVIAC_IMAGE.GetSemiControl: TControl;
begin
  Result:=self;
end;


function TNVIAC_Image.GetTop: integer;
begin
  Result:=inherited Top;
end;

function TNVIAC_IMAGE.GetVisible: boolean;
begin
  Result:=inherited Visible;
end;

function TNVIAC_Image.GetWidth: integer;
begin
  Result:=inherited Width;
end;

function TNVIAC_IMAGE.IsVisible: Boolean;
begin
  Result:=inherited Visible;
end;

procedure TNVIAC_Image.Loaded;
begin
  inherited Loaded;
  SizedShowing;
end;

procedure TNVIAC_IMAGE.Paint;
begin
  if assigned(AImage) and Visible then
    begin
      Canvas.Draw(0,0,AImage);
    end;
end;

procedure TNVIAC_Image.PictureChanged(Sender: TObject);
begin
  if AutoSize and (Picture.Width > 0) and (Picture.Height > 0) then
  inherited SetBounds(Left, Top, Picture.Width, Picture.Height);
  SizedShowing;
//  if not FDrawing then Invalidate;
end;

procedure TNVIAC_Image.Progress(Sender: TObject; Stage: TProgressStage;
  PercentDone: Byte; RedrawNow: Boolean; const R: TRect;
  const Msg: string);
begin
  if FIncrementalDisplay and RedrawNow then
  begin
    if DoPaletteChange then Update
    else Paint;
  end;
  if Assigned(FOnProgress) then FOnProgress(Sender, Stage, PercentDone, RedrawNow, R, Msg);
end;

procedure TNVIAC_IMAGE.ReintroduceBasis(ANewBasis: TNVIAC_BASIS);
begin
  ABasis:=ANewBasis;
end;

procedure TNVIAC_IMAGE.SetAutoSize(Value: Boolean);
begin
  inherited AutoSize:=Value;
end;

procedure TNVIAC_IMAGE.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  SizedHiding;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  SizedShowing;
end;



procedure TNVIAC_Image.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    PictureChanged(Self);
    SizedShowing;
  end;
end;

procedure TNVIAC_Image.SetColor(Value: TColor);
begin
 if (inherited Color) <> Value then
  begin
    inherited Color := Value;
    SizedShowing;
  end;
end;

procedure TNVIAC_IMAGE.SetCurrRect(ARect: TRect);
begin
  ACurrRect.Left:=ARect.Left;
  ACurrRect.Top:=ARect.Top;
  ACurrRect.Right:=ARect.Right;
  ACurrRect.Bottom:=ARect.Bottom;
end;

procedure TNVIAC_IMAGE.SetEnabled1(Value: boolean);
begin
  inherited Enabled:=Value;
  SizedShowing;
end;


procedure TNVIAC_Image.SetHeight(Value: integer);
begin
  inherited Height:=Value;

end;

procedure TNVIAC_Image.SetLeft(Value: integer);
begin
  inherited Left:=Value;
end;

procedure TNVIAC_Image.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
  SizedShowing;
end;

procedure TNVIAC_IMAGE.SetSemiTransparence(Value: Byte);
begin
  if Value<>FSemiTransparence then
    begin
      FSemiTransparence := Value;
      SizedShowing;
    end;
end;


procedure TNVIAC_Image.SetStretch(Value: Boolean);
begin
  if Value <> FStretch then
  begin
    FStretch := Value;
    PictureChanged(Self);
    SizedShowing;
  end;
end;

procedure TNVIAC_Image.SetTop(Value: integer);
begin
  inherited Top:=Value;

end;

procedure TNVIAC_Image.SetTransparent(Value: Boolean);
begin
  if Value<>FTransparent then
    begin
      FTransparent:=Value;
      if Value then
        begin

        end else
        begin
          if assigned(AAlphaImage) then
            begin
              AAlphaImage.Free;
              AAlphaImage:=nil;
            end;
        end;
      SizedShowing;
    end;
end;

procedure TNVIAC_Image.SetTranspRegPower(Value: Byte);
begin
  if Value<>FTranspRegPower then
    begin
      FTranspRegPower:=Value;
      if FTransparent then  SizedShowing;
    end;
end;

procedure TNVIAC_IMAGE.SetVisible(Value: boolean);
begin
  if Value<>(inherited Visible) then
    begin
      inherited Visible:=Value;
      if Value then
        begin
          SizedShowing;
        end else
        begin
          SizedHiding;
        end;
    end;
end;

procedure TNVIAC_Image.SetWidth(Value: integer);
begin
  inherited Width:=Value;
end;

procedure TNVIAC_IMAGE.SizedHiding;
begin
  if assigned(ABasis) and Assigned(AImage) and assigned(Parent) and parent.handleallocated
    and (not (csLoading in ComponentState))then
    begin
      AImage.Width:=Width;
      AImage.Height:=Height;
      ABasis.SemiIsHiding(self);
    end;
end;

procedure TNVIAC_IMAGE.SizedShowing;
begin
  if assigned(Parent) and parent.handleallocated and (not (csLoading in ComponentState))then GetBaseBitmap;
end;

procedure TNVIAC_IMAGE.TryPaint;
begin
  if assigned(Parent) and parent.handleallocated then Repaint;
end;

procedure TNVIAC_IMAGE.TryRegisterSemiQueque;
begin
  if assigned(ABasis) then ABasis.AddToSemiQueue(self);
end;

procedure TNVIAC_IMAGE.TryUnregisterSemiQueque;
begin
  if assigned(ABasis) then ABasis.RemoveFromSemiQueue(self);
end;

end.
