unit NVIAC_Label_Unit;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,NVIAC_BASIS,nviac_textlines_unit,
  nviac_bitmap_unit;

type
  TNVIAC_Label_Style =(nviaclsSimple,nviaclsTransparent,nviaclsDissolve);
  TNVIAC_Label = Class (TGraphicControl,INVIAC_SEMITRANSPARENCE)
  private
    ABasis : TNVIAC_BASIS;
    AImage : TBitmap;
    AAlphaImage : TBitmap;
    FCaptionAngle: integer;
    FCaption3DStyle: TNVIAC_3DStyle;
    FCaptionAlign: TNVIAC_H_Align;
    FCaptionLines: TStringList;
    FSemiTransparence: Byte;
    ACurrRect : TRect;
    FCaptionShadow: Boolean;
    FCaptionShadowColor: TColor;
    FLabelStyle: TNVIAC_Label_Style;

    procedure GetBaseBitmap;
    procedure DefineAutoSize;
    procedure FontIsChanged(Sender: TObject);
    function GetEnabled1: boolean;
    procedure SetEnabled1(Value: boolean);
    function GetVisible: boolean;
    procedure SetVisible(Value: boolean);
    function GetText: TCaption;
    procedure SetText(Value: TCaption);
    function GetAutoSize: Boolean;
    function GetColor: TColor;
    function GetFont: TFont;
    procedure SetAutoSize(Value: Boolean);
    procedure SetColor(Value: TColor);
    procedure SetFont(Value: TFont);
    procedure SetCaption3DStyle(Value: TNVIAC_3DStyle);
    procedure SetCaptionAlign(Value: TNVIAC_H_Align);
    procedure SetCaptionAngle(Value: integer);
    procedure SetCaptionLines(Value: TStringList);
    procedure SetSemiTransparence(Value: Byte);
    function GetHeight: integer;
    function GetLeft: integer;
    function GetTop: integer;
    function GetWidth: integer;
    procedure SetHeight(Value: integer);
    procedure SetLeft(Value: integer);
    procedure SetTop(Value: integer);
    procedure SetWidth(Value: integer);
    procedure SetCaptionShadow(Value: Boolean);
    procedure SetCaptionShadowColor(Value: TColor);
    procedure SetLabelStyle(Value: TNVIAC_Label_Style);
    procedure SizedUpdating;
    procedure SizedHiding;



  protected
    procedure Paint; override;
    procedure Loaded; override; 
  public
    procedure ReintroduceBasis(ANewBasis : TNVIAC_BASIS);
    procedure TryRegisterSemiQueque;
    procedure TryUnregisterSemiQueque;
    function GetFullBitmap: TBitmap;
    function GetCurrentSize : TSize;
    function GetCurrentBitmap: TBitmap;
    function GetCurrentOffset: TPoint;
    function GetCurrentRegion: HRGN;
    function GetSemiClass : Byte;//1 - Alpha,2 - bitmap
    function GetAlpha : Byte;
    function GetAlphaBitmap : TBitmap;
    procedure ContinueBitmap;
    procedure TryPaint;
    procedure SetCurrRect(ARect : TRect);
    function GetSemiControl: TControl;
    function IsVisible: Boolean;


    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer);override;
    procedure Assign(Source: TPersistent);override;
  published
    property AutoSize : Boolean read GetAutoSize write SetAutoSize;
    property Caption : TCaption read GetText write SetText;
    property Color : TColor read GetColor write SetColor;
    property Font : TFont read GetFont write SetFont;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;

    property Enabled : boolean read GetEnabled1 write SetEnabled1;

    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property ShowHint;
    property Visible : boolean read GetVisible write SetVisible;
    property Left: integer read GetLeft write SetLeft;
    property Top: integer read GetTop write SetTop;
    property Width: integer read GetWidth write SetWidth;
    property Height: integer read GetHeight write SetHeight;
    //---defined properties---
    property CaptionLines : TStringList read FCaptionLines write SetCaptionLines;
    property CaptionAlign : TNVIAC_H_Align read FCaptionAlign write SetCaptionAlign default nviachLeft;
    property CaptionAngle : integer read FCaptionAngle write SetCaptionAngle default 0;
    property Caption3DStyle : TNVIAC_3DStyle read FCaption3DStyle write SetCaption3DStyle default nviac3dsSimple;
    property SemiTransparence : Byte read FSemiTransparence write SetSemiTransparence default 0;
    property CaptionShadowColor : TColor read FCaptionShadowColor write SetCaptionShadowColor default clBlack;
    property CaptionShadow : Boolean read FCaptionShadow write SetCaptionShadow;
    property LabelStyle : TNVIAC_Label_Style read FLabelStyle write SetLabelStyle default nviaclsSimple;
  end;
implementation

{ TNVIAC_Label }

procedure TNVIAC_Label.Assign(Source: TPersistent);

begin
  if Source is TNVIAC_Label then
    begin
      //(Source as TNVIAC_Label)
      inherited AutoSize:=(Source as TNVIAC_Label).AutoSize;

      inherited Color:=(Source as TNVIAC_Label).Color;
      inherited Font:=(Source as TNVIAC_Label).Font;
      inherited Caption:=(Source as TNVIAC_Label).Caption;
      FCaptionLines.Text:=(Source as TNVIAC_Label).FCaptionLines.Text;
      ParentBiDiMode:=(Source as TNVIAC_Label).ParentBiDiMode;
      ParentColor:=(Source as TNVIAC_Label).ParentColor;
      ParentFont:=(Source as TNVIAC_Label).ParentFont;
      ParentShowHint:=(Source as TNVIAC_Label).ParentShowHint;
      Align:=(Source as TNVIAC_Label).Align;
      Anchors:=(Source as TNVIAC_Label).Anchors;
      BiDiMode:=(Source as TNVIAC_Label).BiDiMode;
      Constraints:=(Source as TNVIAC_Label).Constraints;
      ShowHint:=(Source as TNVIAC_Label).ShowHint;
      inherited Width:=(Source as TNVIAC_Label).Width;
      inherited Height:=(Source as TNVIAC_Label).Height;

      FCaptionAlign:=(Source as TNVIAC_Label).FCaptionAlign;
      FCaptionAngle:=(Source as TNVIAC_Label).FCaptionAngle;
      FCaption3DStyle:=(Source as TNVIAC_Label).FCaption3DStyle;
      FSemiTransparence:=(Source as TNVIAC_Label).FSemiTransparence;
      FCaptionShadowColor:=(Source as TNVIAC_Label).FCaptionShadowColor;
      FCaptionShadow:=(Source as TNVIAC_Label).FCaptionShadow;
      FLabelStyle:=(Source as TNVIAC_Label).FLabelStyle;
    end else
    begin
      inherited Assign(Source);
    end;
end;

constructor TNVIAC_Label.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not Is_NVIAC_BASIS_We_Got(Self,ABasis) then
    begin
      Reintroduce_NVIAC_BASIS(Self);
      Is_NVIAC_BASIS_We_Got(Self,ABasis);
    end;

    TryRegisterSemiQueque;
    ControlStyle:=ControlStyle+[csOpaque];
    FLabelStyle:=nviaclsSimple;
    FCaptionShadow:=False;
    FCaptionShadowColor:=clBlack;
    ACurrRect:=Rect(0,0,0,0);
    FSemiTransparence:=0;
    inherited Caption:=ClassName;
    FCaptionLines:=TStringList.Create;
    FCaptionLines.Add(ClassName);
    FCaptionLines.OnChange:=FontIsChanged;

    inherited Font.Name:='Arial';
    inherited Font.OnChange:=FontIsChanged;
    FCaptionAlign:=nviachLeft;
    FCaptionAngle:=0;
    FCaption3DStyle:=nviac3dsSimple;
    inherited AutoSize:=True;
    DefineAutoSize;
    AImage:=TBitmap.Create;
    AImage.PixelFormat:=pf24Bit;
    AImage.Width:=inherited Width;
    AImage.Height:=Inherited Height;
    AAlphaImage:=TBitmap.Create;
    AAlphaImage.PixelFormat:=pf24Bit;
end;

procedure TNVIAC_Label.DefineAutoSize;
var b1 : Tbitmap;
    ADefSize : TSize;
begin
//Here we define sizes...
if assigned(Parent) and parent.HandleAllocated then
  begin
    ADefSize:=NVIAC_TextSizeAtAngle(FCaptionLines,Font,Canvas,FCaptionAngle);
  end else
  begin
    B1:=TBitmap.Create;
    B1.Width:=32;
    B1.Height:=32;
    B1.PixelFormat:=pf24bit;
    ADefSize:=NVIAC_TextSizeAtAngle(FCaptionLines,Font,B1.Canvas,FCaptionAngle);
    B1.free;
  end;
 ADefSize.cx:=ADefSize.cx+4;
 ADefSize.cy:=ADefSize.cy+4;
 inherited SetBounds(inherited Left,inherited Top,ADefSize.cx,ADefSize.cy);
end;

destructor TNVIAC_Label.Destroy;
begin
  TryUnregisterSemiQueque;
  AAlphaImage.Free;
  AImage.Free;
  FCaptionLines.Free;
  inherited Destroy;
end;

procedure TNVIAC_Label.FontIsChanged(Sender: TObject);
begin
  if inherited Caption<>FCaptionLines.Text then inherited Caption:=FCaptionLines.Text;
  SizedUpdating;
end;

function TNVIAC_Label.GetAlpha: Byte;
begin
  result:=FSemiTransparence;
end;

function TNVIAC_Label.GetAlphaBitmap: TBitmap;
var ASize : TSize;
    B1 : TBitmap;
    ASin,ACos : Double;
    P1,P2,P3 : array [0..3] of TPoint;
    dw,dh,i : integer;
begin
if assigned(Parent) and parent.HandleAllocated then
  begin
    ASize:=NVIAC_TextSize(FCaptionLines,Font,Canvas);
  end else
  begin
    B1:=TBitmap.Create;
    B1.Width:=32;
    B1.Height:=32;
    B1.PixelFormat:=pf24bit;
    ASize:=NVIAC_TextSize(FCaptionLines,Font,B1.Canvas);
    B1.free;
  end;
  ASin:=Sin(FCaptionAngle*pi/1800);
  ACos:=Cos(FCaptionAngle*pi/1800);
  dw:=round(abs(ACos*ASize.cx)+abs(ASin*ASize.cy));
  dh:=round(abs(ASin*ASize.cx)+abs(ACos*ASize.cy));
  if ((ASin>0) and (ACos>0)) or ((ASin<0) and (ACos<0)) then
    begin
      P1[0].x:=0;
      P1[0].y:=Round(ASize.cx*abs(ASin));
      P1[1].x:=Round(ASize.cx*abs(ACos));
      P1[1].y:=0;
      P1[2].x:=dw;
      P1[2].y:=Round(ASize.cy*abs(ACos));
      P1[3].x:=Round(ASize.cy*abs(ASin));
      P1[3].y:=dh;
    end else
    begin
      P1[0].x:=0;
      P1[0].y:=Round(ASize.cy*abs(ACos));
      P1[1].x:=Round(ASize.cy*abs(ASin));
      P1[1].y:=0;
      P1[2].x:=dw;
      P1[2].y:=Round(ASize.cx*abs(ASin));
      P1[3].x:=Round(ASize.cx*abs(ACos));
      P1[3].y:=dh;
    end;
  for i:=0 to 3 do
    begin
      P1[i].x:=P1[i].x+2;
      P1[i].y:=P1[i].y+2;
      P2[i].x:=P1[i].x;
      P2[i].y:=P1[i].y;
      P3[i].x:=P1[i].x;
      P3[i].y:=P1[i].y;
      if P1[i].x=2 then P1[i].x:=0;
      if P1[i].y=2 then P1[i].y:=0;
      if P1[i].x=dw+2 then P1[i].x:=P1[i].x+2;
      if P1[i].y=dh+2 then P1[i].y:=P1[i].y+2;
      if P3[i].x=2 then P3[i].x:=4;
      if P3[i].y=2 then P3[i].y:=4;
      if P3[i].x=dw+2 then P3[i].x:=P3[i].x-2;
      if P3[i].y=dh+2 then P3[i].y:=P3[i].y-2;
    end;




  AAlphaImage.Width:=AImage.Width;
  AAlphaImage.Height:=AImage.Height;
  AAlphaImage.Canvas.Brush.Color:=clWhite;
  AAlphaImage.Canvas.FillRect(Rect(0,0,AAlphaImage.Width,AAlphaImage.height));
  case FLabelStyle of
    nviaclsTransparent :
      begin
        AAlphaImage.Canvas.Brush.Color:=RGB(FSemiTransparence,FSemiTransparence,FSemiTransparence);
        AAlphaImage.Canvas.Pen.Style:=psClear;
        AAlphaImage.Canvas.Polygon(P1);
        AAlphaImage.Canvas.Pen.Style:=psSolid;
      end;
    nviaclsDissolve:
      begin
        AAlphaImage.Canvas.Brush.Color:=GetCColor(RGB(FSemiTransparence,FSemiTransparence,FSemiTransparence),clWhite,70);
        AAlphaImage.Canvas.Pen.Style:=psClear;
        AAlphaImage.Canvas.Polygon(P1);
        AAlphaImage.Canvas.Pen.Style:=psClear;
        AAlphaImage.Canvas.Brush.Color:=GetCColor(RGB(FSemiTransparence,FSemiTransparence,FSemiTransparence),clWhite,30);
        AAlphaImage.Canvas.Polygon(P2);
        AAlphaImage.Canvas.Pen.Style:=psClear;
        AAlphaImage.Canvas.Brush.Color:=RGB(FSemiTransparence,FSemiTransparence,FSemiTransparence);
        AAlphaImage.Canvas.Polygon(P3);
        AAlphaImage.Canvas.Pen.Style:=psSolid;
      end;
  end;


  result:=AAlphaImage;
end;

function TNVIAC_Label.GetAutoSize: Boolean;
begin
  result:=inherited AutoSize;
end;

procedure TNVIAC_Label.GetBaseBitmap;
begin
  //here we will got the bitmap with semi....
 if assigned(AImage) and (inherited Width>0) and (inherited Height>0) then
    begin
      AImage.Width:=inherited Width;
      AImage.Height:=inherited Height;
      if Assigned(ABasis) then ABasis.SemiIsUpdated(self);
    end;
end;

function TNVIAC_Label.GetText: TCaption;
begin
  result:=Inherited Caption;
end;

function TNVIAC_Label.GetColor: TColor;
begin
  result:=inherited Color;
end;

function TNVIAC_Label.GetCurrentBitmap: TBitmap;
var ARect : TRect;
    ARgn: HRGN;
    AColor : TColor;
    AFont : TFont;
begin
  if assigned(AImage) and (Width>0) and (Height>0) then
    begin
      intersectrect(ARect,ACurrRect,Rect(0,0,Width,Height));
      if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,Width,Height);
      ARGN:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);

      AImage.Width:=Width;
      AImage.Height:=Height;

      SelectClipRGN(AImage.Canvas.Handle,ARGN);

      AImage.Canvas.Brush.Color:=Color;
      AImage.Canvas.FillRect(rect(0,0,AImage.Width,AImage.Height));
      AImage.Canvas.Brush.Style:=bsClear;
      AColor:=Color;
      AFont:=TFont.Create;
      AFont.Assign(Font);
      if FCaptionShadow then
        begin
          AFont.Color:=GetCColor(AColor,FCaptionShadowColor,30);
          NVIAC_DrawLinesAtAngle(FCaptionLines,AImage.Canvas,AFont,FCaptionAngle,4,4,FCaptionAlign);

        end;
      case FCaption3DStyle of
         nviac3dsPressed :
          begin
            AFont.Color:=GetCColor(AColor,clWhite,30);
            NVIAC_DrawLinesAtAngle(FCaptionLines,AImage.Canvas,AFont,FCaptionAngle,3,3,FCaptionAlign);
            AFont.Color:=GetCColor(AColor,clBlack,30);
            NVIAC_DrawLinesAtAngle(FCaptionLines,AImage.Canvas,AFont,FCaptionAngle,1,1,FCaptionAlign);
          end;
        nviac3dsRised :
          begin
            AFont.Color:=GetCColor(AColor,clBlack,30);
            NVIAC_DrawLinesAtAngle(FCaptionLines,AImage.Canvas,AFont,FCaptionAngle,3,3,FCaptionAlign);
            AFont.Color:=GetCColor(AColor,clWhite,30);
            NVIAC_DrawLinesAtAngle(FCaptionLines,AImage.Canvas,AFont,FCaptionAngle,1,1,FCaptionAlign);
          end;
      end;
      AFont.Free;
      AImage.Canvas.Brush.Style:=bsSolid;
      SelectClipRGN(AImage.Canvas.Handle,0);
      DeleteObject(ARGN);
    end;
  result:=AImage;    
end;

function TNVIAC_Label.GetCurrentOffset: TPoint;
var APos1 : TSize;
begin
  APos1:=GetControlPosition(Self);
  Result.x:=APos1.cx;
  Result.y:=APos1.cy;
end;

function TNVIAC_Label.GetCurrentRegion: HRGN;
var ARect : TRect;
begin
  intersectrect(ARect,ACurrRect,Rect(0,0,AImage.Width,AImage.Height));
  if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,AImage.Width,AImage.Height);
  Result:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);
end;

function TNVIAC_Label.GetCurrentSize: TSize;
begin
  if assigned(AImage) then
    begin
      result.cx:=AImage.Width;
      result.cy:=AImage.Height;
    end else
    begin
      result.cx:=0;
      result.cy:=0;
    end;
end;

function TNVIAC_Label.GetEnabled1: boolean;
begin
  result:=inherited Enabled;
end;

function TNVIAC_Label.GetFont: TFont;
begin
  result:=inherited Font;
end;

function TNVIAC_Label.GetFullBitmap: TBitmap;
begin
  Result:=AImage;//defined by context.... before bitmap definition or after
end;

function TNVIAC_Label.GetSemiClass: Byte;
begin
  case FLabelStyle of
    nviaclsSimple : result:=1;
    nviaclsTransparent,nviaclsDissolve: result:=2;
    else result:=1;
  end;

end;

function TNVIAC_Label.GetVisible: boolean;
begin
  result:=inherited Visible;
end;

procedure TNVIAC_Label.Paint;
begin
  if assigned(AImage) then
    begin
      Canvas.Draw(0,0,AImage);
    end;
end;

procedure TNVIAC_Label.SetAutoSize(Value: Boolean);
begin
  inherited AutoSize:=Value;
  if Value then
    begin
      SizedHiding;
      DefineAutoSize;
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  SizedHiding;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  GetBaseBitmap;
  invalidate;
end;

procedure TNVIAC_Label.SetText(Value: TCaption);
begin
  SizedHiding;
  inherited Caption:=Value;
  if FCaptionLines.Text<>Value then FCaptionLines.Text:=Value;
  SizedUpdating;
end;

procedure TNVIAC_Label.SetCaption3DStyle(Value: TNVIAC_3DStyle);
begin
  if Value<>FCaption3DStyle then
    begin
      FCaption3DStyle := Value;
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.SetCaptionAlign(Value: TNVIAC_H_Align);
begin
  if Value<>FCaptionAlign then
    begin
      FCaptionAlign := Value;
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.SetCaptionAngle(Value: integer);
begin
while Value>1800 do dec(Value,3600);
while Value<=-1800 do inc(Value,3600);
  if Value<>FCaptionAngle then
    begin
      FCaptionAngle := Value;
      if inherited AutoSize then
        begin
          SizedHiding;
          DefineAutoSize;
          GetBaseBitmap;
          invalidate;
        end else
        begin
          GetBaseBitmap;
          invalidate;
        end;
    end;
end;

procedure TNVIAC_Label.SetCaptionLines(Value: TStringList);
begin
  if assigned(Value) then
    begin
      if FCaptionLines.Text=Value.Text then exit;
      FCaptionLines.Assign(Value);

    end else
    begin
      FCaptionLines.Clear;

    end;

  if inherited AutoSize then
    begin
      SizedHiding;
      DefineAutoSize;
      GetBaseBitmap;
      invalidate;
    end else
    begin
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.SetColor(Value: TColor);
begin
  inherited Color:=Value;
  GetBaseBitmap;
  invalidate;
end;



procedure TNVIAC_Label.SetEnabled1(Value: boolean);
begin
  inherited Enabled:=Value;
  GetBaseBitmap;
  invalidate;
end;

procedure TNVIAC_Label.SetFont(Value: TFont);
begin
  inherited Font:=Value;
  SizedUpdating;
end;

procedure TNVIAC_Label.SetVisible(Value: boolean);
begin
  inherited Visible:=Value;
  if Value then
    begin
      GetBaseBitmap;
      invalidate;
    end else
    begin
      SizedHiding;
    end;
end;

procedure TNVIAC_Label.TryRegisterSemiQueque;
begin
  if assigned(ABasis) then ABasis.AddToSemiQueue(self);
end;

procedure TNVIAC_Label.TryUnregisterSemiQueque;
begin
  if assigned(ABasis) then ABasis.RemoveFromSemiQueue(self);
end;

procedure TNVIAC_Label.SetSemiTransparence(Value: Byte);
begin
  if Value<>FSemiTransparence then
    begin
      FSemiTransparence := Value;
      GetBaseBitmap;
      invalidate;
    end;
end;


procedure TNVIAC_Label.ContinueBitmap;
var ARect : TRect;
    ARgn: HRGN;
    AFont : TFont;
begin
// here we continue to get the bitmap after semi...

  if assigned(AImage) and (AImage.Width>0) and (AImage.Height>0) then
    begin
      intersectrect(ARect,ACurrRect,Rect(0,0,AImage.Width,AImage.Height));
      if IsRectEmpty(ACurrRect) then ARect:=Rect(0,0,AImage.Width,AImage.Height);
      ARGN:=CreateRectRGN(ARect.Left,ARect.Top,ARect.Right,ARect.Bottom);
      AImage.Canvas.Brush.Color:=clYellow;
      SelectClipRGN(AImage.Canvas.Handle,ARGN);
      //Drawing
      AImage.Canvas.Brush.Style:=bsClear;
      AFont:=TFont.Create;
      AFont.Assign(Font);

      if Enabled then
        begin
          AFont.Color:=Font.Color;
        end else
        begin
          AFont.Color:=GetCColor(Font.Color,Color,50);
        end;
      NVIAC_DrawLinesAtAngle(FCaptionLines,AImage.Canvas,AFont,FCaptionAngle,2,2,FCaptionAlign);
      AImage.Canvas.Brush.Style:=bsSolid;
      AFont.Free;

      SelectClipRGN(AImage.Canvas.Handle,0);
      DeleteObject(ARGN);
    end;
end;

procedure TNVIAC_Label.SetCurrRect(ARect: TRect);
begin
  ACurrRect.Left:=ARect.Left;
  ACurrRect.Top:=ARect.Top;
  ACurrRect.Right:=ARect.Right;
  ACurrRect.Bottom:=ARect.Bottom;
end;

procedure TNVIAC_Label.ReintroduceBasis(ANewBasis: TNVIAC_BASIS);
begin
  ABasis:=ANewBasis;
end;

function TNVIAC_Label.GetHeight: integer;
begin
  result:=inherited Height;
end;

function TNVIAC_Label.GetLeft: integer;
begin
  result:=inherited Left;
end;

function TNVIAC_Label.GetTop: integer;
begin
  result:=inherited Top;
end;

function TNVIAC_Label.GetWidth: integer;
begin
  result:=inherited Width;
end;

procedure TNVIAC_Label.SetHeight(Value: integer);
begin
  inherited Height:=Value;
end;

procedure TNVIAC_Label.SetLeft(Value: integer);
begin
  inherited Left:=Value;
end;

procedure TNVIAC_Label.SetTop( Value: integer);
begin
  inherited Top:=Value;
end;

procedure TNVIAC_Label.SetWidth(Value: integer);
begin
  inherited Width:=Value;
end;

procedure TNVIAC_Label.SetCaptionShadow(Value: Boolean);
begin
  if Value<>FCaptionShadow then
    begin
      FCaptionShadow := Value;
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.SetCaptionShadowColor(Value: TColor);
begin
  if Value<>FCaptionShadowColor then
    begin
      FCaptionShadowColor := Value;
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.SetLabelStyle( Value: TNVIAC_Label_Style);
begin
  if Value<>FLabelStyle then
    begin
      FLabelStyle := Value;
      GetBaseBitmap;
      invalidate;
    end;
end;

procedure TNVIAC_Label.TryPaint;
begin
  Repaint;// invalidate;// repaint;
end;

function TNVIAC_Label.GetSemiControl: TControl;
begin
  Result:=Self;
end;

procedure TNVIAC_Label.SizedUpdating;
begin
  if inherited AutoSize then DefineAutoSize;
  GetBaseBitmap;
  invalidate;
end;

procedure TNVIAC_Label.SizedHiding;
begin
  if assigned(ABasis) and Assigned(AImage) and assigned(Parent) and parent.handleallocated then
  begin
    AImage.Width:=Width;
    AImage.Height:=Height;
    ABasis.SemiIsHiding(self);
  end;
end;

function TNVIAC_Label.IsVisible: Boolean;
begin
  Result:=Visible;
end;




procedure TNVIAC_Label.Loaded;
begin
  inherited Loaded;
  if inherited AutoSize then
    begin
      SizedHiding;
      DefineAutoSize;
      GetBaseBitmap;
      invalidate;
    end else
    begin
      GetBaseBitmap;
      invalidate;
    end;
end;

end.
